@@ -26,7 +26,8 @@ __all__ = [
     'list_all_funnels', 'list_all_remote_process_groups', 'delete_funnel',
     'get_remote_process_group', 'update_process_group', 'create_funnel',
     'create_remote_process_group', 'delete_remote_process_group',
-    'set_remote_process_group_transmission', 'get_pg_parents_ids'
+    'set_remote_process_group_transmission', 'get_pg_parents_ids',
+    'delete_port', 'create_port'
 ]
 
 log = logging.getLogger(__name__)
@@ -142,7 +143,8 @@ def get_process_group(identifier, identifier_type='name', greedy=True):
             out = nipyapi.nifi.ProcessGroupsApi().get_process_group(identifier)
         else:
             obj = list_all_process_groups()
-            out = nipyapi.utils.filter_obj(obj, identifier, identifier_type, greedy=greedy)
+            out = nipyapi.utils.filter_obj(
+                obj, identifier, identifier_type, greedy=greedy)
     return out
 
 
@@ -214,7 +216,7 @@ def list_invalid_processors(pg_id='root', summary=False):
     assert isinstance(pg_id, six.string_types), "pg_id should be a string"
     assert isinstance(summary, bool)
     proc_list = [x for x in list_all_processors(pg_id)
-                 if x.component.validation_errors ]
+                 if x.component.validation_errors]
     if summary:
         out = [{'id': x.id, 'summary': x.component.validation_errors}
                for x in proc_list]
@@ -1168,7 +1170,7 @@ def delete_controller(controller, force=False):
     return result
 
 
-def update_controller(controller, update, refresh=True):
+def update_controller(controller, update):
     """
     Updates the Configuration of a Controller Service
 
@@ -1176,7 +1178,6 @@ def update_controller(controller, update, refresh=True):
         controller (ControllerServiceEntity): Target Controller to update
         update (ControllerServiceDTO): Controller Service configuration object
             containing the new config params and properties
-        refresh (bool): True to refresh before applying
 
     Returns:
         (ControllerServiceEntity)
@@ -1185,8 +1186,6 @@ def update_controller(controller, update, refresh=True):
     assert isinstance(controller, nipyapi.nifi.ControllerServiceEntity)
     assert isinstance(update, nipyapi.nifi.ControllerServiceDTO)
     # Insert the ID into the update
-    if refresh:
-        controller = get_controller(controller.id, 'id')
     update.id = controller.id
     return nipyapi.nifi.ControllerServicesApi().update_controller_service(
         id=controller.id,
@@ -1257,7 +1256,7 @@ def schedule_controller(controller, scheduled, refresh=False):
     raise ValueError("Scheduling request timed out")
 
 
-def get_controller(identifier, identifier_type='name', bool_response=False, greedy=True):
+def get_controller(identifier, identifier_type='name', bool_response=False):
     """
     Retrieve a given Controller
 
@@ -1266,7 +1265,6 @@ def get_controller(identifier, identifier_type='name', bool_response=False, gree
         identifier_type (str): 'id' or 'name', defaults to name
         bool_response (bool): If True, will return False if the Controller is
             not found - useful when testing for deletion completion
-        greedy (bool): True for partial match, False for exact match
 
     Returns:
 
@@ -1279,7 +1277,7 @@ def get_controller(identifier, identifier_type='name', bool_response=False, gree
             out = handle.get_controller_service(identifier)
         else:
             obj = list_all_controllers()
-            out = nipyapi.utils.filter_obj(obj, identifier, identifier_type, greedy=greedy)
+            out = nipyapi.utils.filter_obj(obj, identifier, identifier_type)
     except nipyapi.nifi.rest.ApiException as e:
         if bool_response:
             return False
@@ -1533,13 +1531,13 @@ def create_funnel(pg_id, position=None):
         return nipyapi.nifi.ProcessGroupsApi().create_funnel(
             id=pg_id,
             body=nipyapi.nifi.FunnelEntity(
-                position=nipyapi.nifi.PositionDTO(
-                    x=float(position[0]),
-                    y=float(position[1])
-                ),
                 revision=nipyapi.nifi.RevisionDTO(version=0),
                 component=nipyapi.nifi.FunnelDTO(
-                    parent_group_id=pg_id
+                    parent_group_id=pg_id,
+                    position=nipyapi.nifi.PositionDTO(
+                        x=float(position[0]),
+                        y=float(position[1])
+                    ),
                 )
             )
         )
