@@ -1,5 +1,13 @@
+# -*- coding: utf-8 -*-
+
+"""
+A self-paced walkthrough of version control using NiFi-Registry.
+See initial print statement for detailed explanation.
+"""
+
 from __future__ import absolute_import
 import logging
+from time import sleep
 import nipyapi
 from nipyapi.utils import DockerContainer
 
@@ -56,8 +64,9 @@ d_containers = [
     )
 ]
 
-dev_pg_name = 'dev_pg_0'
-dev_proc_name = 'dev_proc_0'
+dev_pg_name = 'my_pg_0'
+dev_proc_name = 'my_proc_0'
+dev_proc2_name = 'my_s_proc_0'
 dev_reg_client_name = 'dev_reg_client_0'
 dev_bucket_name = 'dev_bucket_0'
 dev_ver_flow_name = 'dev_ver_flow_0'
@@ -74,16 +83,17 @@ print("This python script demonstrates the steps to manage promotion of "
       "in hex (0,1,2,3,4,5,6,7,8,9,a,b,c,d) and should be called in order."
       "\nEach step will log activities to INFO, and you are encouraged to "
       "look at the code in this script to see how each step is completed."
-      "\nEach step will also issue instructions through print statements like"
+      "\nhttp://github.com/Chaffelson/nipyapi/blob/master/nipyapi/demo/fdlc.py"
+      "\nEach step will also issue instructions through print statements like "
       "this one, these instructions will vary so please read them as you go."
       "\nNote that the first call will log a lot of information while it boots"
-      "the Docker containers, further instructions will follow."
+      " the Docker containers, further instructions will follow."
       "\nNote that you can reset it at any time by calling step_1 again.\n"
-      "\nPlease start by calling the function 'step_1_boot_demo_env()'."
-      )
+      "\nPlease start by calling the function 'step_1_boot_demo_env()'.")
 
 
 def step_1_boot_demo_env():
+    """step_1_boot_demo_env"""
     log.info("Starting Dev and Prod NiFi and NiFi-Registry Docker Containers"
              "\nPlease wait, this may take a few minutes to download the "
              "Docker images and then start them.")
@@ -109,6 +119,8 @@ def step_1_boot_demo_env():
             nipyapi_delay=nipyapi.config.long_retry_delay,
             nipyapi_max_wait=nipyapi.config.long_max_wait
         )
+        # Sleeping to wait for all startups to return before printing guide
+        sleep(1)
     print("Your Docker containers should now be ready, please find them at the"
           "following URLs:"
           "\nnifi-dev   ", dev_nifi_url,
@@ -116,12 +128,11 @@ def step_1_boot_demo_env():
           "\nreg-prod   ", prod_reg_url,
           "\nnifi-prod  ", prod_nifi_url,
           "\nPlease open each of these in a browser tab."
-          "\nPlease then call the function 'step_2_create_reg_clients()'\n"
-          )
+          "\nPlease then call the function 'step_2_create_reg_clients()'\n")
 
 
 def step_2_create_reg_clients():
-    # Set client connections between NiFi and Registry
+    """Set client connections between NiFi and Registry"""
     log.info("Creating Dev Environment Nifi to NiFi-Registry Client")
     nipyapi.utils.set_endpoint(dev_nifi_api_url)
     nipyapi.versioning.create_registry_client(
@@ -145,6 +156,7 @@ def step_2_create_reg_clients():
 
 
 def step_3_create_dev_flow():
+    """Connecting to Dev environment and creating some test objects"""
     log.info("Connecting to Dev environment and creating some test objects")
     nipyapi.utils.set_endpoint(dev_nifi_api_url)
     nipyapi.utils.set_endpoint(dev_reg_api_url)
@@ -177,6 +189,7 @@ def step_3_create_dev_flow():
 
 
 def step_4_create_dev_ver_bucket():
+    """Creating dev registry bucket"""
     log.info("Creating %s as new a Registry Bucket", dev_bucket_name)
     nipyapi.versioning.create_registry_bucket(dev_bucket_name)
     print("We have created a new Versioned Flow Bucket in the Dev "
@@ -187,6 +200,7 @@ def step_4_create_dev_ver_bucket():
 
 
 def step_5_save_flow_to_bucket():
+    """Saving the flow to the bucket as a new versioned flow"""
     log.info(
         "Saving %s to %s", dev_pg_name, dev_bucket_name)
     process_group = nipyapi.canvas.get_process_group(dev_pg_name)
@@ -211,6 +225,7 @@ def step_5_save_flow_to_bucket():
 
 
 def step_6_export_dev_flow():
+    """Exporting the versioned flow as a yaml definition"""
     log.info("Creating a sorted pretty Yaml export of %s",
              dev_flow_export_name)
     bucket = nipyapi.versioning.get_registry_bucket(dev_bucket_name)
@@ -234,6 +249,7 @@ def step_6_export_dev_flow():
 
 
 def step_7_create_prod_ver_bucket():
+    """Connecting to the Prod environment and creating a new bucket"""
     log.info("Connecting to Prod Environment")
     nipyapi.utils.set_endpoint(prod_nifi_api_url)
     nipyapi.utils.set_endpoint(prod_reg_api_url)
@@ -248,6 +264,7 @@ def step_7_create_prod_ver_bucket():
 
 
 def step_8_import_dev_flow_to_prod_reg(versioned_flow):
+    """Importing the yaml string into Prod"""
     log.info("Saving dev flow export to prod container")
     bucket = nipyapi.versioning.get_registry_bucket(prod_bucket_name)
     nipyapi.versioning.import_flow_version(
@@ -263,6 +280,7 @@ def step_8_import_dev_flow_to_prod_reg(versioned_flow):
 
 
 def step_9_deploy_prod_flow_to_nifi():
+    """Deploying the flow to the Prod environment"""
     log.info("Deploying promoted flow from Prod Registry to Prod Nifi")
     bucket = nipyapi.versioning.get_registry_bucket(prod_bucket_name)
     flow = nipyapi.versioning.get_flow_in_bucket(
@@ -272,7 +290,7 @@ def step_9_deploy_prod_flow_to_nifi():
     reg_client = nipyapi.versioning.get_registry_client(prod_reg_client_name)
     nipyapi.versioning.deploy_flow_version(
         parent_id=nipyapi.canvas.get_root_pg_id(),
-        location=(0,0),
+        location=(0, 0),
         bucket_id=bucket.identifier,
         flow_id=flow.identifier,
         reg_client_id=reg_client.id,
@@ -288,6 +306,7 @@ def step_9_deploy_prod_flow_to_nifi():
 
 
 def step_a_change_dev_flow():
+    """Procedurally modifying the Dev flow"""
     log.info("Connecting to Dev Environment")
     nipyapi.utils.set_endpoint(dev_nifi_api_url)
     nipyapi.utils.set_endpoint(dev_reg_api_url)
@@ -308,6 +327,7 @@ def step_a_change_dev_flow():
 
 
 def step_b_update_dev_flow_ver():
+    """Committing the change to the dev flow version"""
     log.info("Saving changes in Dev Flow to Version Control")
     process_group = nipyapi.canvas.get_process_group(dev_pg_name)
     bucket = nipyapi.versioning.get_registry_bucket(dev_bucket_name)
@@ -333,6 +353,7 @@ def step_b_update_dev_flow_ver():
 
 
 def step_c_promote_change_to_prod_reg():
+    """Promoting the committed change across to the prod environment"""
     log.info("Exporting updated Dev Flow Version")
     dev_bucket = nipyapi.versioning.get_registry_bucket(dev_bucket_name)
     dev_ver_flow = nipyapi.versioning.get_flow_in_bucket(
@@ -368,6 +389,7 @@ def step_c_promote_change_to_prod_reg():
 
 
 def step_d_promote_change_to_prod_nifi():
+    """Pushing the change into the Prod flow"""
     log.info("Moving deployed Prod Process Group to the latest version")
     prod_pg = nipyapi.canvas.get_process_group(dev_pg_name)
     nipyapi.versioning.update_flow_ver(
@@ -378,5 +400,138 @@ def step_d_promote_change_to_prod_nifi():
           "of being up to date with its version control."
           "\nLook at the Processor scheduling to note that it now matches the "
           "dev environment as 3s."
-          "\nThis is the end of the demo, you may restart it by calling"
-          "'step_1_boot_demo_env() at any time.\n")
+          "\nNow we will examine some typical deployment tests."
+          "\nPlease now call 'step_e_check_sensitive_processors()'\n")
+
+
+def step_e_check_sensitive_processors():
+    """Create and test for Sensitive Properties to be set in the Canvas"""
+    log.info("Connecting to Dev Environment")
+    nipyapi.utils.set_endpoint(dev_nifi_api_url)
+    nipyapi.utils.set_endpoint(dev_reg_api_url)
+    log.info("Creating additional complex Processor")
+    nipyapi.canvas.create_processor(
+        parent_pg=nipyapi.canvas.get_process_group(dev_pg_name),
+        processor=nipyapi.canvas.get_processor_type('GetTwitter'),
+        location=(400.0, 600.0),
+        name=dev_proc2_name,
+    )
+    s_proc = nipyapi.canvas.list_sensitive_processors(summary=True)
+    print("We have created a new Processor {0} which has security protected"
+          "properties, these will need to be completed in each environment "
+          "that this flow is used in. These properties are discoverable using "
+          "the API calls list 'canvas.list_sensitive_processors()'"
+          "\nFunction 'nipyapi.canvas.update_processor' as used in step_a is"
+          " intended for this purpose"
+          "\nPlease no call 'step_f_set_sensitive_values()'\n"
+          .format(s_proc[0].status.name, ))
+
+
+def step_f_set_sensitive_values():
+    """Set the Sensitive Properties to pass the deployment test"""
+    log.info("Setting Sensitive Values on Processor")
+    nipyapi.canvas.update_processor(
+        processor=nipyapi.canvas.get_processor(dev_proc2_name),
+        update=nipyapi.nifi.ProcessorConfigDTO(
+            properties={
+                'Consumer Key': 'Some',
+                'Consumer Secret': 'Secret',
+                'Access Token': 'values',
+                'Access Token Secret': 'here'
+            }
+        )
+    )
+    print("Here we have set the Sensitive values, again using the Update"
+          " process. Typically these values will be looked up in a Config DB "
+          "or some other secured service."
+          "\nPlease now call 'step_g_check_invalid_processors()'\n")
+    # Todo: update sensitive to return properites list precreated
+
+
+def step_g_check_invalid_processors():
+    """Look for Processors with Invalid Properties to be remediated"""
+    log.info("Retrieving Processors in Invalid States")
+    i_proc = nipyapi.canvas.list_invalid_processors()[0]
+    print("We now run a validity test against our flow to ensure that it can "
+          "be deployed. We can see that Processors [{0}] need further "
+          "attention."
+          "\nWe can also easily see the reasons for this [{1}]."
+          "\nPlease now call 'step_h_fix_validation_errors()'\n"
+          .format(i_proc.status.name, i_proc.component.validation_errors))
+
+
+def step_h_fix_validation_errors():
+    """Update values for Invalid Processors to pass the deployment test"""
+    log.info("Autoterminating Success status")
+    nipyapi.canvas.update_processor(
+        processor=nipyapi.canvas.get_processor(dev_proc2_name),
+        update=nipyapi.nifi.ProcessorConfigDTO(
+            auto_terminated_relationships=['success']
+        )
+    )
+    print("We now see that our Processor is configured and Valid within this "
+          "environment, and is ready for Promotion to the next stage."
+          "\nPlease now call 'step_i_promote_deploy_and_validate()'\n")
+
+
+def step_i_promote_deploy_and_validate():
+    """Promotion and Deployment in a single method"""
+    log.info("Saving changes in Dev Flow to Version Control")
+    dev_process_group = nipyapi.canvas.get_process_group(dev_pg_name)
+    dev_bucket = nipyapi.versioning.get_registry_bucket(dev_bucket_name)
+    dev_registry_client = nipyapi.versioning.get_registry_client(
+        dev_reg_client_name)
+    dev_flow = nipyapi.versioning.get_flow_in_bucket(
+        bucket_id=dev_bucket.identifier,
+        identifier=dev_ver_flow_name
+    )
+    nipyapi.versioning.save_flow_ver(
+        process_group=dev_process_group,
+        registry_client=dev_registry_client,
+        bucket=dev_bucket,
+        flow_id=dev_flow.identifier,
+        comment='A Flow update with a Complex Processor'
+    )
+    dev_ver_flow = nipyapi.versioning.get_flow_in_bucket(
+        dev_bucket.identifier,
+        identifier=dev_ver_flow_name
+    )
+    log.info("Exporting the Dev flow to Yaml")
+    dev_export = nipyapi.versioning.export_flow_version(
+        bucket_id=dev_bucket.identifier,
+        flow_id=dev_ver_flow.identifier,
+        mode='yaml'
+    )
+    log.info("Connecting to Prod Environment")
+    nipyapi.utils.set_endpoint(prod_nifi_api_url)
+    nipyapi.utils.set_endpoint(prod_reg_api_url)
+    log.info("Importing the Updated Dev Yaml to the Prod Bucket Flow")
+    prod_bucket = nipyapi.versioning.get_registry_bucket(prod_bucket_name)
+    prod_flow = nipyapi.versioning.get_flow_in_bucket(
+        bucket_id=prod_bucket.identifier,
+        identifier=prod_ver_flow_name
+    )
+    nipyapi.versioning.import_flow_version(
+        bucket_id=prod_bucket.identifier,
+        encoded_flow=dev_export,
+        flow_id=prod_flow.identifier
+    )
+    log.info("Pushing the new Version into the Prod Flow")
+    prod_pg = nipyapi.canvas.get_process_group(dev_pg_name)
+    nipyapi.versioning.update_flow_ver(
+        process_group=prod_pg,
+        target_version=None
+    )
+    log.info("Checking for Invalid Processors in the new flow")
+    val_errors = nipyapi.canvas.list_invalid_processors()
+    print("Here we have put all the steps in one place by taking the dev "
+          "changes all the way through to prod deployment. If we check"
+          " our Processor Validation again, we see that our regular "
+          "Properties have been carried through, but our Sensitive "
+          "Properties are unset in Production [{0}]"
+          "\nThis is because NiFi will not break"
+          " security by carrying them to a new environment. We leave setting"
+          " them again as an exercise for the user."
+          .format(val_errors[0].component.validation_errors))
+    print("\nThis is the end of the guide, you may restart at any time by "
+          "calling 'step_1_boot_demo_env()'\n")
