@@ -44,7 +44,7 @@ def list_service_users(service='nifi'):
 
 def get_service_user(identifier, identifier_type='identity', service='nifi'):
     """
-    Filters the all users list for a given identifier and type
+    Gets the unique user matching to the given identifier and type.
 
     Args:
         identifier (str): the string to search for
@@ -52,14 +52,16 @@ def get_service_user(identifier, identifier_type='identity', service='nifi'):
         service (str): the name of the service
 
     Returns:
-        None if no match, list of multiple matches, else single object
+        None if no match, else single object
 
     """
     assert service in _valid_services
     assert isinstance(identifier, six.string_types)
     assert isinstance(identifier_type, six.string_types)
     obj = list_service_users(service)
-    out = nipyapi.utils.filter_obj(obj, identifier, identifier_type)
+    out = nipyapi.utils.filter_obj(
+        obj, identifier, identifier_type, greedy=False
+    )
     return out
 
 
@@ -155,26 +157,34 @@ def create_service_user_group(identity, service='nifi',
     """
     assert service in _valid_services
     assert isinstance(identity, six.string_types)
+
+    users_ids = None
+
     if service == 'nifi':
-        assert all(isinstance(user, nipyapi.nifi.UserEntity) for user in users)
-    else:
-        assert all(isinstance(user, nipyapi.registry.User) for user in users)
-    if service == 'registry':
-        user_group_obj = nipyapi.registry.UserGroup(
-            identity=identity,
-            users=[{'identifier': user.identifier} for user in users]
-        )
-    else:
-        # must be nifi
+        if users:
+            assert all(
+                isinstance(user, nipyapi.nifi.UserEntity) for user in users
+            )
+            users_ids = [{'id': user.id} for user in users]
         user_group_obj = nipyapi.nifi.UserGroupEntity(
             revision=nipyapi.nifi.RevisionDTO(
                 version=0
             ),
             component=nipyapi.nifi.UserGroupDTO(
                 identity=identity,
-                users=[{'id': user.id} for user in users]
+                users=users_ids
             )
         )
+    else:
+        if users:
+            assert all(
+                isinstance(user, nipyapi.registry.User) for user in users
+            )
+            users_ids = [{'identifier': user.identifier} for user in users]
+        user_group_obj = nipyapi.registry.UserGroup(
+            identity=identity,
+            users=users_ids
+        )
     try:
         return getattr(nipyapi, service).TenantsApi().create_user_group(
             user_group_obj
@@ -208,7 +218,7 @@ def list_service_user_groups(service='nifi'):
 def get_service_user_group(identifier, identifier_type='identity',
                            service='nifi'):
     """
-    Filters the all groups list for a given identifier and type
+    Gets the unique group matching to the given identifier and type.
 
     Args:
         identifier (str): the string to search for
@@ -216,14 +226,15 @@ def get_service_user_group(identifier, identifier_type='identity',
         service (str): the name of the service
 
     Returns:
-        None if no match, list of multiple matches, else single object
+        None if no match, else single object
 
     """
     assert service in _valid_services
     assert isinstance(identifier, six.string_types)
     assert isinstance(identifier_type, six.string_types)
     obj = list_service_user_groups(service)
-    out = nipyapi.utils.filter_obj(obj, identifier, identifier_type)
+    out = nipyapi.utils.filter_obj(
+        obj, identifier, identifier_type, greedy=False)
     return out
 
 
@@ -316,7 +327,7 @@ def service_login(service='nifi', username=None, password=None,
     configuration.username = uname
     configuration.password = pword
     if auth_type == 'token':
-        log.info("Attempting login with user identity [%s]",
+        log.info("Attempting tokenAuth login with user identity [%s]",
                  configuration.username)
         try:
             if service == 'nifi':
@@ -331,7 +342,7 @@ def service_login(service='nifi', username=None, password=None,
             raise ValueError(e.body)
         set_service_auth_token(token=token, service=service)
     elif auth_type == 'basic':
-        pass
+        log.info("basicAuth set, skipping token retrieval")
     return True
 
 
@@ -379,7 +390,14 @@ def service_logout(service='nifi'):
     """
     assert service in _valid_services
     set_service_auth_token(token=None, service=service)
-    status = get_service_access_status(service, bool_response=True)
+    try:
+        status = get_service_access_status(service, bool_response=True)
+    except ValueError as e:
+        if 'Cannot set verify_mode to CERT_NONE' in str(e):
+            status = None
+            # Logout throws error with incorrect ssl setup
+        else:
+            raise e
     # Set to empty string and not None as basic auth setup will still
     # run even if not used
     getattr(nipyapi, service).configuration.password = ''
